@@ -1,3 +1,4 @@
+import base64
 import logging
 from datetime import datetime
 from typing import Union, List, Dict
@@ -14,6 +15,17 @@ from ..utils.payload import generate_payload
 logger = logging.getLogger('Ts.Client')
 
 
+class TsClientApi(BaseAbcp):
+    def __init__(self, *args):
+        super().__init__(*args)
+        self.good_receipts = GoodReceipts(*args)
+        self.order_pickings = OrderPickings(*args)
+        self.customer_complaints = CustomerComplaints(*args)
+        self.orders = Orders(*args)
+        self.cart = Cart(*args)
+        self.positions = Positions(*args)
+
+
 class GoodReceipts(BaseAbcp):
     async def create(self, code: Union[str, int],
                      positions: Union[List[Dict[str, str]], Dict[str, str]],
@@ -33,7 +45,7 @@ class GoodReceipts(BaseAbcp):
         payload = generate_payload(**locals())
         if isinstance(sup_shipment_date, datetime):
             sup_shipment_date = f'{sup_shipment_date:%Y-%m-%d %H:%M:%S}'
-        return await self.request(api.Methods.TsClient.GoodReceipts.CREATE, payload, True)
+        return await self._request(api.Methods.TsClient.GoodReceipts.CREATE, payload, True)
 
     async def get(self, limit: int = None, skip: int = None,
                   output: str = None,
@@ -62,7 +74,7 @@ class GoodReceipts(BaseAbcp):
         :param sup_number:
         :return:
         """
-        if isinstance(limit, int) and not (1 <= limit <= 1000):
+        if isinstance(limit, int) and not 1 <= limit <= 1000:
             raise AbcpWrongParameterError('Параметр "limit" может принимать значения от 1 до 1000')
         if isinstance(date_start, datetime):
             date_start = generate(date_start.replace(tzinfo=pytz.utc))
@@ -76,7 +88,7 @@ class GoodReceipts(BaseAbcp):
             else:
                 raise AbcpWrongParameterError('Параметр "statuses" принимет значения от 1 до 3')
         payload = generate_payload(**locals())
-        return await self.request(api.Methods.TsClient.GoodReceipts.GET, payload)
+        return await self._request(api.Methods.TsClient.GoodReceipts.GET, payload)
 
     async def get_positions(self, op_id: Union[str, int], limit: int = None, skip: int = None,
                             output: str = None, product_id: Union[int, str] = None, auto: str = None):
@@ -93,14 +105,14 @@ class GoodReceipts(BaseAbcp):
         :param auto:
         :return:
         """
-        if isinstance(limit, int) and not (1 <= limit <= 1000):
+        if isinstance(limit, int) and not 1 <= limit <= 1000:
             raise AbcpWrongParameterError('Параметр "limit" может принимать значения от 1 до 1000')
         if isinstance(output, str) and output != 'e':
             raise AbcpWrongParameterError('Параметр "output" принимает только значение "e"')
         if isinstance(auto, str) and (len(auto) < 3):
             raise AbcpWrongParameterError('Параметр "auto" должен состоять минимум из 3 символов')
         payload = generate_payload(**locals())
-        return await self.request(api.Methods.TsClient.GoodReceipts.GET_POSITIONS, payload)
+        return await self._request(api.Methods.TsClient.GoodReceipts.GET_POSITIONS, payload)
 
 
 class OrderPickings(BaseAbcp):
@@ -130,13 +142,13 @@ class OrderPickings(BaseAbcp):
         :param co_old_pos_ids: список идентификаторов позиций старых заказов
         :return:
         """
-        if isinstance(limit, int) and not (1 <= limit <= 1000):
+        if isinstance(limit, int) and not 1 <= limit <= 1000:
             raise AbcpWrongParameterError('Параметр "limit" может принимать значения от 1 до 1000')
         if isinstance(date_start, datetime):
             date_start = generate(date_start.replace(tzinfo=pytz.utc))
         if isinstance(date_end, datetime):
             date_end = generate(date_end.replace(tzinfo=pytz.utc))
-        if isinstance(status, int) and not (1 <= status <= 3):
+        if isinstance(status, int) and not 1 <= status <= 3:
             raise AbcpWrongParameterError('Параметр "status" принимет значения от 1 до 3')
         if isinstance(status, list):
             if all(1 <= x <= 3 for x in status):
@@ -146,7 +158,7 @@ class OrderPickings(BaseAbcp):
         if isinstance(output, str) and (not all(x in 'des' for x in output) or len(output) > 3):
             raise AbcpWrongParameterError('Параметр "output" должен состоять из  ["d", "e", "s"]')
         payload = generate_payload(**locals())
-        return await self.request(api.Methods.TsClient.OrderPickings.GET, payload)
+        return await self._request(api.Methods.TsClient.OrderPickings.GET, payload)
 
     async def get_positions(self, op_id: Union[str, int], limit: int = None, skip: int = None, output: str = None,
                             product_id: Union[int, str] = None,
@@ -181,10 +193,10 @@ class OrderPickings(BaseAbcp):
 
         if isinstance(output, str) and (not all(x in 'oe' for x in output) or len(output) > 2):
             raise AbcpWrongParameterError('Параметр "output" должен состоять из  ["o", "e"]')
-        elif output is not None:
+        if output is not None:
             raise AbcpWrongParameterError('output must be a string')
         payload = generate_payload(**locals())
-        return await self.request(api.Methods.TsClient.OrderPickings.GET_POSITIONS, payload)
+        return await self._request(api.Methods.TsClient.OrderPickings.GET_POSITIONS, payload)
 
 
 class CustomerComplaints(BaseAbcp):
@@ -229,14 +241,14 @@ class CustomerComplaints(BaseAbcp):
             date_start = generate(date_start.replace(tzinfo=pytz.utc))
         if isinstance(date_end, datetime):
             date_end = generate(date_end.replace(tzinfo=pytz.utc))
-        if isinstance(tag_ids, int) or isinstance(tag_ids, str):
+        if isinstance(tag_ids, (int, str)):
             tag_ids = [tag_ids]
         if isinstance(position_statuses, list):
             position_statuses = ','.join(map(str, position_statuses))
         if fields is not None:
             fields = check_fields(fields, self.FieldsChecker.get_fields)
         payload = generate_payload(**locals())
-        return await self.request(api.Methods.TsClient.CustomerComplaints.GET, payload)
+        return await self._request(api.Methods.TsClient.CustomerComplaints.GET, payload)
 
     async def get_positions(self, op_id: Union[str, int],
                             order_picking_good_id: Union[int, str] = None,
@@ -286,9 +298,9 @@ class CustomerComplaints(BaseAbcp):
             date_start = generate(date_start.replace(tzinfo=pytz.utc))
         if isinstance(date_end, datetime):
             date_end = generate(date_end.replace(tzinfo=pytz.utc))
-        if isinstance(status, int) and not (1 <= status <= 6):
+        if isinstance(status, int) and not 1 <= status <= 6:
             raise AbcpWrongParameterError('Параметр "status" должен быть в диапазоне от 1 до 6')
-        if isinstance(type, int) and not (1 <= type <= 3):
+        if isinstance(type, int) and not 1 <= type <= 3:
             raise AbcpWrongParameterError('Параметр "type" должен быть в диапазоне от 1 до 3')
         if isinstance(output, str) and output != 'e':
             raise AbcpWrongParameterError('Параметр "output" принимает только значение "e"')
@@ -296,7 +308,7 @@ class CustomerComplaints(BaseAbcp):
             fields = check_fields(fields, self.FieldsChecker.get_positions_fields)
 
         payload = generate_payload(**locals())
-        return await self.request(api.Methods.TsClient.CustomerComplaints.GET_POSITIONS, payload)
+        return await self._request(api.Methods.TsClient.CustomerComplaints.GET_POSITIONS, payload)
 
     async def create(self, order_picking_id: Union[str, int], positions: Union[List[Dict], Dict]):
         """
@@ -312,7 +324,21 @@ class CustomerComplaints(BaseAbcp):
         if isinstance(positions, dict):
             positions = [positions]
         payload = generate_payload(exclude=['positions'], **locals())
-        return await self.request(api.Methods.TsClient.CustomerComplaints.CREATE, payload, True)
+        return await self._request(api.Methods.TsClient.CustomerComplaints.CREATE, payload, True)
+
+    async def create_position_multiple(self, positions: Union[List[Dict], Dict],
+                                       customer_complaint_id: int,
+                                       customer_complaint: str,
+                                       custom_complaint_file: str = None):
+        with open(custom_complaint_file, "rb") as ccf:
+            encoded_string = base64.b64encode(ccf.read()).decode("utf-8")
+        custom_complaint_file = f"{encoded_string}"
+        del ccf
+        del encoded_string
+        if isinstance(positions, dict):
+            positions = [positions]
+        payload = generate_payload(**locals())
+        return await self._request(api.Methods.TsClient.CustomerComplaints.CREATE_POSITION_MULTIPLE, payload, True)
 
     async def update_position(self, id: int, quantity: Union[str, int]):
         """
@@ -327,7 +353,7 @@ class CustomerComplaints(BaseAbcp):
         :return:
         """
         payload = generate_payload(**locals())
-        return await self.request(api.Methods.TsClient.CustomerComplaints.UPDATE, payload, True)
+        return await self._request(api.Methods.TsClient.CustomerComplaints.UPDATE, payload, True)
 
     async def cancel_position(self, id: int):
         """
@@ -342,7 +368,7 @@ class CustomerComplaints(BaseAbcp):
         :return:
         """
         payload = generate_payload(**locals())
-        return await self.request(api.Methods.TsClient.CustomerComplaints.CANCEL, payload, True)
+        return await self._request(api.Methods.TsClient.CustomerComplaints.CANCEL, payload, True)
 
 
 class Orders(BaseAbcp):
@@ -364,13 +390,13 @@ class Orders(BaseAbcp):
         """
         if isinstance(create_time, datetime):
             create_time = generate(create_time.replace(tzinfo=pytz.utc))
-        if isinstance(positions, int) or isinstance(positions, str):
+        if isinstance(positions, (int, str)):
             positions = [positions]
         payload = generate_payload(
             exclude=['delivery_address', 'delivery_person',
                      'delivery_contact', 'delivery_comment', 'delivery_method_id'],
             **locals())
-        return await self.request(api.Methods.TsClient.Orders.CREATE, payload, True)
+        return await self._request(api.Methods.TsClient.Orders.CREATE, payload, True)
 
     async def get_list(self, number: Union[str, int] = None, agreement_id: Union[int, str] = None,
                        manager_id: Union[int, str] = None,
@@ -423,7 +449,7 @@ class Orders(BaseAbcp):
         if isinstance(deadline_date_end, datetime):
             deadline_date_end = generate(deadline_date_end.replace(tzinfo=pytz.utc))
         payload = generate_payload(**locals())
-        return await self.request(api.Methods.TsClient.Orders.GET_LIST, payload)
+        return await self._request(api.Methods.TsClient.Orders.GET_LIST, payload)
 
     async def get_order(self, order_id: Union[str, int]):
         """
@@ -435,7 +461,7 @@ class Orders(BaseAbcp):
         :return:
         """
         payload = generate_payload(**locals())
-        return await self.request(api.Methods.TsClient.Orders.GET, payload)
+        return await self._request(api.Methods.TsClient.Orders.GET, payload)
 
     async def refuse(self, order_id: Union[str, int]):
         """
@@ -447,7 +473,7 @@ class Orders(BaseAbcp):
         :return:
         """
         payload = generate_payload(**locals())
-        return await self.request(api.Methods.TsClient.Orders.REFUSE, payload, True)
+        return await self._request(api.Methods.TsClient.Orders.REFUSE, payload, True)
 
 
 class Cart(BaseAbcp):
@@ -467,7 +493,7 @@ class Cart(BaseAbcp):
         :return:
         """
         payload = generate_payload(**locals())
-        return await self.request(api.Methods.TsClient.Cart.CREATE, payload, True)
+        return await self._request(api.Methods.TsClient.Cart.CREATE, payload, True)
 
     async def update(self, position_id: Union[str, int], quantity: int):
         """
@@ -480,7 +506,7 @@ class Cart(BaseAbcp):
         :return:
         """
         payload = generate_payload(**locals())
-        return await self.request(api.Methods.TsClient.Cart.UPDATE, payload, True)
+        return await self._request(api.Methods.TsClient.Cart.UPDATE, payload, True)
 
     async def get_list(self, position_ids: Union[List, str, int] = None, agreement_id: Union[int, str] = None,
                        limit: int = None,
@@ -498,7 +524,7 @@ class Cart(BaseAbcp):
         if isinstance(position_ids, list):
             position_ids = ','.join(map(str, position_ids))
         payload = generate_payload(**locals())
-        return await self.request(api.Methods.TsClient.Cart.GET_LIST, payload, True)
+        return await self._request(api.Methods.TsClient.Cart.GET_LIST, payload, True)
 
     async def exist(self, agreement_id: Union[str, int], brand: str, number_fix: str):
         """
@@ -512,7 +538,7 @@ class Cart(BaseAbcp):
         :return:
         """
         payload = generate_payload(**locals())
-        return await self.request(api.Methods.TsClient.Cart.EXIST, payload)
+        return await self._request(api.Methods.TsClient.Cart.EXIST, payload)
 
     async def summary(self, agreement_id: Union[int, str] = None):
         """
@@ -523,7 +549,7 @@ class Cart(BaseAbcp):
         :return:
         """
         payload = generate_payload(**locals())
-        return await self.request(api.Methods.TsClient.Cart.SUMMARY, payload)
+        return await self._request(api.Methods.TsClient.Cart.SUMMARY, payload)
 
     async def clear(self, agreement_id: Union[str, int]):
         """
@@ -534,7 +560,7 @@ class Cart(BaseAbcp):
         :return:
         """
         payload = generate_payload(**locals())
-        return await self.request(api.Methods.TsClient.Cart.CLEAR, payload, True)
+        return await self._request(api.Methods.TsClient.Cart.CLEAR, payload, True)
 
     async def delete_positions(self, position_ids: Union[List, str, int]):
         """
@@ -544,11 +570,11 @@ class Cart(BaseAbcp):
         :param position_ids:
         :return:
         """
-        if isinstance(position_ids, int) or isinstance(position_ids, str):
+        if isinstance(position_ids, (int, str)):
             position_ids = [position_ids]
 
         payload = generate_payload(**locals())
-        return await self.request(api.Methods.TsClient.Cart.DELETE, payload, True)
+        return await self._request(api.Methods.TsClient.Cart.DELETE, payload, True)
 
 
 class Positions(BaseAbcp):
@@ -568,7 +594,7 @@ class Positions(BaseAbcp):
         if additional_info is not None:
             additional_info = check_fields(additional_info, self.FieldsChecker.additional_info)
         payload = generate_payload(**locals())
-        return await self.request(api.Methods.TsClient.Positions.GET, payload)
+        return await self._request(api.Methods.TsClient.Positions.GET, payload)
 
     async def get_list(self, brand: str = None, message: str = None, agreement_id: Union[int, str] = None,
                        manager_id: Union[int, str] = None,
@@ -621,15 +647,15 @@ class Positions(BaseAbcp):
             deadline_date_start = f'{deadline_date_start:%Y-%m-%d %H:%M:%S}'
         if isinstance(deadline_date_end, datetime):
             deadline_date_end = f'{deadline_date_end:%Y-%m-%d %H:%M:%S}'
-        if isinstance(product_ids, int) or isinstance(product_ids, str):
+        if isinstance(product_ids, (int, str)):
             product_ids = [product_ids]
-        if isinstance(route_ids, int) or isinstance(route_ids, str):
+        if isinstance(route_ids, (int, str)):
             route_ids = [route_ids]
-        if isinstance(distributor_ids, int) or isinstance(distributor_ids, str):
+        if isinstance(distributor_ids, (int, str)):
             distributor_ids = [distributor_ids]
-        if isinstance(ids, int) or isinstance(ids, str):
+        if isinstance(ids, (int, str)):
             ids = [ids]
-        if isinstance(order_ids, int) or isinstance(order_ids, str):
+        if isinstance(order_ids, (int, str)):
             order_ids = [order_ids]
         if isinstance(statuses, str):
             statuses = [statuses]
@@ -640,7 +666,7 @@ class Positions(BaseAbcp):
         if additional_info is not None:
             additional_info = check_fields(additional_info, self.FieldsChecker.additional_info)
         payload = generate_payload(**locals())
-        return await self.request(api.Methods.TsClient.Positions.GET_LIST, payload)
+        return await self._request(api.Methods.TsClient.Positions.GET_LIST, payload)
 
     async def cancel(self, position_id: Union[str, int], additional_info: Union[List, str] = None):
         """
@@ -652,20 +678,8 @@ class Positions(BaseAbcp):
         if additional_info is not None:
             additional_info = check_fields(additional_info, self.FieldsChecker.additional_info)
         payload = generate_payload(**locals())
-        return await self.request(api.Methods.TsClient.Positions.CANCEL, payload, True)
+        return await self._request(api.Methods.TsClient.Positions.CANCEL, payload, True)
 
     async def mass_cancel(self, position_ids: Union[List, str, int], additional_info: Union[List, str] = None):
         payload = generate_payload(**locals())
-        return await self.request(api.Methods.TsClient.Positions.MASS_CANCEL, payload, True)
-
-
-class TsClientApi(BaseAbcp):
-    def __init__(self, *args):
-        super().__init__(*args)
-        # If you know how do it other way please commit on https://github.com/bl4ckm45k/aioabcpapi
-        self.good_receipts = GoodReceipts(*args)
-        self.order_pickings = OrderPickings(*args)
-        self.customer_complaints = CustomerComplaints(*args)
-        self.orders = Orders(*args)
-        self.cart = Cart(*args)
-        self.positions = Positions(*args)
+        return await self._request(api.Methods.TsClient.Positions.MASS_CANCEL, payload, True)
